Package: distory
Version: 1.4.5
Date: 2024-10-07
Title: Distance Between Phylogenetic Histories
Authors@R: c(person(given = "John", family = "Chakerian", role = c("aut")),
   person(given = "Susan", family = "Holmes", role = c("aut")),
   person(given = "Emmanuel", family = "Paradis", role = "cre",
          email = "Emmanuel.Paradis@ird.fr"))
Depends: ape (>= 5.0)
Imports: graphics, stats
Description: Geodesic distance between phylogenetic trees and
  associated functions. The theoretical background of 'distory'
  is published in Billera et al. (2001) "Geometry of the space of
  phylogenetic trees." <doi:10.1006/aama.2001.0759>.
License: BSD_3_clause + file LICENSE
NeedsCompilation: yes
Packaged: 2024-10-07 19:48:04 UTC; paradis
Author: John Chakerian [aut],
  Susan Holmes [aut],
  Emmanuel Paradis [cre]
Maintainer: Emmanuel Paradis <Emmanuel.Paradis@ird.fr>
Repository: CRAN
Date/Publication: 2024-10-07 21:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 00:25:58 UTC; unix
Archs: distory.so.dSYM

useDynLib(distory, .registration = TRUE)

export(bethe.tree, bin.multiPhylo, boot.samples.idxs,
       convert.table.to.idx, dist.multiPhylo, distinct.edges,
       edge.from.split, get.bipartition, gromov.hyperbolicity,
       lookup.samples, mcmc.target.seq, orthant.boundary.tree,
       partition.leaves, phylo.diff, position.leverage)

importFrom(graphics, par, plot)
importFrom(stats, as.dist, runif)
import(ape)

gromov.hyperbolicity <- function(d, deltas = FALSE, scale = NA)
{
    d <- as.matrix(as.dist(d))
    a = dim(d);
    if(a[1] != a[2])
    {
        stop("The parameter could not be coerced into a square distance matrix.")
    }

    if(a[1] < 4)
    {
        stop("At least 4 points must be used to compute the Gromov hyperbolicity constant.")
    }

    if(is.na(scale))
    {
        scale = "none"
    }

    scaleV = pmatch(scale, c("none", "max", "perimeter"))

    if(length(scaleV) > 1 || is.na(scaleV))
    {
        stop("Unknown or ambiguous scale method.");
    }

    .Call(gromov_distmatrix, as.double(d), as.logical(deltas),
          as.integer(scaleV))
}

bethe.tree <- function(tips, level.lengths = NULL, outgroup="O", outgroup.dist=1)
{
    if(length(tips) != length(unique(tips)))
    {
        stop("Not all tips are unique.\n")
    }

    if(!is.null(level.lengths))
    {
        if(length(level.lengths) == 1)
        {
            level.lengths = rep(level.lengths, 2)
        }
    }

    if(is.null(level.lengths) || length(level.lengths) == 0)
    {
        level.lengths = 1
    }

    groupings <- as.numeric(sapply(1:(length(tips)/2), function(x) rep(x,2)))

    d <- level.lengths[1]
    level.lengths <- level.lengths[-1]
    tips <- lapply(split(tips,groupings), function(x) { sprintf("(%s:%d,%s:%d)",x[1],d,x[2],d)} ) 
    if(length(tips) == 1)
    {
        read.tree(text=sprintf("(%s:%d, %s:%d);", tips[[1]], d, outgroup, outgroup.dist))
    }
    else
    {
        bethe.tree(tips, level.lengths, outgroup, outgroup.dist)
    }
}

phylo.diff <- function(x, y, ...)
{
    uniqT1 <- distinct.edges(x, y)
    uniqT2 <- distinct.edges(y, x)
    
    treeA.cs <- rep("black", dim(x$edge)[1]) 
    treeA.cs[uniqT1] = "red"

    treeB.cs <- rep("black", dim(y$edge)[1]) 
    treeB.cs[uniqT2] = "red"

    par(mfrow=c(1,2))
    plot(x, edge.color=treeA.cs, ...)
    plot(y, edge.color=treeB.cs, ...)

    invisible()
}

distinct.edges <- function(x, y) # all edges in x not in y
{
    bp1 <- partition.leaves(x)
    bp1 <- lapply(bp1, sort)
    bp2 <- partition.leaves(y)
    bp2 <- lapply(bp2, sort)

    p = c()

    for(i in 1:length(bp1))
    {
        if(!(list(bp1[[i]]) %in% bp2))
        {
            p <- append(p, i)
        }
    }

    p
}

edge.from.split <- function(x, split)
{
    splits <- partition.leaves(x)
    splits <- lapply(splits, sort)
    split <- sort(split)
    match(list(split), splits)
}

get.bipartition <- function(x, e) # of leaves from edge
{
    acc = vector()
    inc = FALSE;
    for(i in 1:dim(x$edge)[1])
    {
        if(x$edge[i,1] == e)
        {
            acc <- append(acc, get.bipartition(x, x$edge[i,2]));
            inc = TRUE;
        }
    }

    if(!inc)
    {
        acc = x$tip.label[e];
    }

    acc
}

partition.leaves <- function(x) # get all bipartitoins
{
    r = list()
    for(i in 1:dim(x$edge)[1])
    {
        t <- get.bipartition(x, x$edge[i,2]);
        r <- append(r, list(t));
    }

    r
}

orthant.boundary.tree <- function(x, y)
{
    e1 <- distinct.edges(x,y)
    e2 <- distinct.edges(y,x)

    if(length(e1) != 1)
        stop("Trees must differ by only one edge")

    length.e1 = x$edge.length[e1]
    length.e2 = y$edge.length[e2]

    lambda = length.e1 / ( length.e1 + length.e2)

    bdy.tree <- x

    partitions <- partition.leaves(x)

    bdy.tree$edge.length <- lapply(partitions,
            function(e)
            {
                a.e <- x$edge.length[edge.from.split(x, e)]
                b.e <- y$edge.length[edge.from.split(y, e)]

                a.e*(1 - lambda) + b.e * lambda
            }
        )
    
    bdy.tree$edge.length[e1] = 0;

    bdy.tree
}

mcmc.target.seq <- function(data, x, F, n) 
{
    N = ncol(data)
    D = rep(1, N)

    likvals = c()

    a = runif(n)

    old.tree = F(lookup.samples(data, list(convert.table.to.idx(D)))[[1]])
    old.dist = as.matrix(dist.multiPhylo(list(x, old.tree)))[1,2]

    best.tree = old.tree
    best.dist = old.dist

    for(i in 1:n)
    {
        Dp = D
        good = FALSE

        r = sample.int(N, 2)
        while(D[r[1]] == N || D[r[2]] == 0)
        {    
            r = sample.int(N, 2)
        }

        # propose the swap
        Dp[r[1]] = Dp[r[1]] + 1;
        Dp[r[2]] = Dp[r[2]] - 1;

        new.tree = F(lookup.samples(data, list(convert.table.to.idx(Dp)))[[1]])
        new.dist = as.matrix(dist.multiPhylo(list(x, new.tree)))[1,2]

        v = old.dist/new.dist

        if(v >= 1) 
        { 
            v = 1 
            best.tree = new.tree
            best.dist = new.dist
        } 

        if(v^(100-exp(1/i)) > a[i])
        {
            D = Dp
            old.tree = new.tree
            old.dist = new.dist
        }

        likvals = c(likvals, old.dist)
    }
    
    list(data = D, tree = best.tree, distance = best.dist, vals = likvals)
}

# modified from ape code for boot.phylo
boot.samples.idxs <- function(data, B = 100, block = 1)
{
    if (is.list(data) && !is.data.frame(data)) {
        if (inherits(data, "DNAbin")) 
            data <- as.matrix(data)
        else {
            nm <- names(data)
            n <- length(data)
            data <- unlist(data)
            nL <- length(data)
            data <- matrix(data, n, nL/n, byrow = TRUE)
            rownames(data) <- nm
        }
    }
    boot.smpls <- vector("list", B)
    for (i in 1:B) {
        if (block > 1) {
            y <- seq(block, ncol(data), block)
            boot.i <- sample(y, replace = TRUE)
            boot.samp <- numeric(ncol(data))
            boot.samp[y] <- boot.i
            for (j in 1:(block - 1)) boot.samp[y - j] <- boot.i - 
                j
        }
        else boot.samp <- sample(ncol(data), replace = TRUE)

        boot.smpls[[i]] <- boot.samp
    }

    boot.smpls
}

lookup.samples <- function(data, idxs)
{
    if (is.list(data) && !is.data.frame(data)) {
        if (inherits(data, "DNAbin")) 
            data <- as.matrix(data)
        else {
            nm <- names(data)
            n <- length(data)
            data <- unlist(data)
            nL <- length(data)
            data <- matrix(data, n, nL/n, byrow = TRUE)
            rownames(data) <- nm
        }
    }

    lapply(idxs, function(i) { data[,i] } )
}

phylo.diff <- function(x, y, ...)
{
    uniqT1 <- distinct.edges(x, y)
    uniqT2 <- distinct.edges(y, x)
    
    treeA.cs <- rep("black", dim(x$edge)[1]) 
    treeA.cs[uniqT1] = "red"

    treeB.cs <- rep("black", dim(y$edge)[1]) 
    treeB.cs[uniqT2] = "red"

    par(mfrow=c(1,2))
    plot(x, edge.color=treeA.cs, ...)
    plot(y, edge.color=treeB.cs, ...)

    invisible()
}

convert.table.to.idx <- function(T)
{
    dat = c()
    for(i in 1:length(T))
    {
        dat = c(dat, rep(i, T[i]))
    }

    dat
}

position.leverage <- function(data, F, to = NULL, rep = 50, by = 1)
{
    N = ncol(data)
    D = rep(1, N)

    leverage.vals = c()

    if(is.null(to))
    {
        to = F(lookup.samples(data, list(convert.table.to.idx(D)))[[1]])
    }

    for(i in seq(1,N,by))
    {
        Dp = D;
        S = sample(N, N/rep);
        while(i %in% S)
        {
            S = sample(N,N/rep);
        }

        Dp[S] = 0;
        Dp[i] = N/rep;

        new.tree = F(lookup.samples(data, list(convert.table.to.idx(Dp)))[[1]])
        leverage.vals = c(leverage.vals, as.matrix(dist.multiPhylo(list(to,
                            new.tree)))[1,2])
    }

    leverage.vals
}

dist.multiPhylo <- function(x, method="geodesic", force.multi2di = FALSE,
        outgroup = NULL, convert.multifurcating = FALSE, use.random.resolution =
        FALSE, scale = NULL, verbose = FALSE)
{
    if(length(x) < 2)
        return(matrix())

    if(class(x) == "multiPhylo") # ideally, we will have this
    {
        # run checks if appropriate
        # separate it out into a vector of strings

        if(!is.null(outgroup))
        {
            x <- lapply(x, function(k)
                    {
                    if(class(k) == "phylo")
                    {
                        if(!is.rooted(k))
                            root(k, outgroup, resolve.root = TRUE)
                        else k
                    }
                    else NA
                    })
        }


        if(force.multi2di == TRUE)
        {
            x <- lapply(x, function(k)
                  {
                    if(class(k) == "phylo")
                    {
                    multi2di(k,random=use.random.resolution)
                    }
                    else NA
                  })
        }
        else if(convert.multifurcating == TRUE) # won't resolve multifurcations at the root
        {
            x <- lapply(x, function(k)
                  {
                    if(class(k) == "phylo")
                    {
                        if(!is.binary.phylo(k))
                            multi2di(k,random=use.random.resolution)
                        else k
                    }
                    else NA
                  })

        }

        if(!is.null(scale))
        {
            if(class(scale) == "phylo")
            {
                T <- sum(scale$edge.length)
                x <- lapply(x, function(k)
                        {
                            if(class(k) == "phylo")
                            {
                                k$edge.length = k$edge.length * (T /
                                    sum(k$edge.length))
                                k
                            }
                            else NA
                            })
            }
            else if(class(scale) == "numeric")
            {
                x <- lapply(x, function(k)
                        {
                            if(class(k) == "phylo")
                            {
                                k$edge.length = k$edge.length * (scale /
                                    sum(k$edge.length))
                                k
                            }
                            else NA
                            })
            }
            else
            {
                stop("Scale parameter not understood.\n")
            }
        }

        # do some sanity checks before we start out

        r <- lapply(x, function(k)
                {
                if(class(k) == "phylo")
                {
                    is.rooted(k)
                }
                else NA
                })

        if(!all(as.logical(r), na.rm=TRUE))
        {
            stop("Some trees are not rooted. Specify an outgroup to fix this problem. All trees must be rooted.\n")
        }

        r <- lapply(x, function(k) { if(class(k) == "phylo") is.binary.phylo(k) else NA })
        if(!all(as.logical(r), na.rm=TRUE))
        {
            stop("Some trees are not binary. All input trees must be strictly binary.\n")
        }

        # check to make sure all trees have the same tip labels
        tips = x[[1]]$tip.label
        r <- lapply(x, function(k) { if(class(k) == "phylo") setequal(k$tip.label,
                    tips) else NA})

        if(!all(as.logical(r), na.rm=TRUE))
        {
            stop("Not all trees have the same tips.\n")
        }

        # convert our list of class phylo to a list of strings
        treestrs <- lapply(x, function(k) { if(class(k) == "phylo")
                write.tree(k) else "" })

        method=tolower(method)

        method.id = pmatch(method, c("edgeset", "geodesic"))

        # call the C interface function and return the value automatically
        if(method.id == 1)
        {
            rv <- .Call(phycpp_bin_trees, treestrs)
        }
        else if(method.id == 2)
        {
            rv <- .Call(phycpp_compute_tree_distance_set, treestrs, as.logical(verbose))
        }
        else
        {
            stop("Specified method is not valid")
        }

        as.dist(rv)
    }

    else if(typeof(x) == "list")
    {
        if(class(x[[1]]) == "phylo") # a list of phylo's that for some reason is not classed as multiPhylo
        {
            class(x) <- "multiPhylo" # it already is basically a multiPhylo anyways - we'll mark it as such
            dist.multiPhylo(x, method=method, force.multi2di=force.multi2di, outgroup=outgroup,
                        convert.multifurcating=convert.multifurcating,
                        use.random.resolution=use.random.resolution,
                        scale=scale, verbose=verbose)
        }
        else if(class(x[[1]]) == "character") # a list of strings, presuming one tree each, properly terminated
        {
            # read with /ape/, run checks, dump back
            t <- paste(x, sep="", collapse="")
            k <- read.tree(text=t)
            dist.multiPhylo(x, method=method, force.multi2di=force.multi2di, outgroup=outgroup,
                        convert.multifurcating=convert.multifurcating,
                        use.random.resolution=use.random.resolution,
                        scale=scale, verbose=verbose)
        }
    }
    else if(class(x) == "character") # this is for one string containing multiple trees
    {
        # read with ape and dump back to a vector of strings
        k <- read.tree(text=x)

        # call this to process it properly
        dist.multiPhylo(x, method=method, force.multi2di=force.multi2di, outgroup=outgroup,
                        convert.multifurcating=convert.multifurcating,
                        use.random.resolution=use.random.resolution,
                        scale=scale, verbose=verbose)
    }
    else
    {
        stop("Cannot coerce the argument into a usable type.")
    }
}

bin.multiPhylo <- function(treelist)
{
    edge.dists <- dist.multiPhylo(treelist, method="edgeset")
    edge.dists <- as.matrix(edge.dists)

    bin.id = 1
    binning = rep(NA,length(treelist))
    while(any(is.na(binning)))
    {
         # find the next unbinned tree & find all matching trees
         next.tr = min(which(is.na(binning)))
         binning[edge.dists[,next.tr] == 0] = bin.id
         bin.id = bin.id + 1
    }

    binning
}

\name{dist.multiPhylo}
\Rdversion{1.1}
\alias{dist.multiPhylo}
\title{Geodesic Distance Between Phylogenetic Trees}
\description{
  Computes the geodesic distance of a list of phylogenetic trees using a
  polynomial algorithm.
}
\usage{
dist.multiPhylo(x, method = "geodesic", force.multi2di = FALSE,
                outgroup = NULL, convert.multifurcating = FALSE,
                use.random.resolution = FALSE, scale = NULL,
                verbose = FALSE)
}
\arguments{
  \item{x}{
    A list of ape trees (class 'phylo'). The list does not have to be of
    class 'multiPhylo'. The function will also accept a list of strings of
    trees in Newick format, or a single string with trees in Newick format
    separated by semicolons. All the trees must have the same tip labels.}
  \item{method}{
    Determines which distance method is used. Options are 'geodesic' for the
    tree space geodesic distance, or 'edgeset' for the number of edges
    (defined by splits of tips) that are different.}
  \item{force.multi2di}{
    Force conversion of every tree to strict bifurcating through the ape
    function 'multi2di', using the use.random.resolution as its parameter.
    This option should not be used in conjunction with specification of an
    outgroup.}
  \item{outgroup}{
    Specifies an outgroup to root each tree with respect to. This calls the
    ape function 'root' on every tree in the list.}
  \item{convert.multifurcating}{
    Setting this option will check every tree for multifurcations using the
    ape function 'is.binary.tree' - if it returns FALSE, the ape function
    'multi2di' will be called on it. Note that this does not ensure a tree
    is strictly binary, since ape considers an unrooted tree binary even if
    the root node is trifurcating. This option can be used in conjunction
    with specification of an outgroup.}
  \item{use.random.resolution}{
    Specifies the parameter to 'multi2di' if needed.}
  \item{scale}{
    Specifies a scale to make all trees unformly scaled (that is, the sum of
    all edges will be uniform)scale to make all trees unformly scaled (that
    is, the sum of all edge lengths will be uniform). The parameter can
    either be a tree of class \code{phylo} or a numeric value for the sum of
    all edge lengths.}
  \item{verbose}{
    Turns on incremental status updates and more warnings. Helpful for large
    computations.}
}
\details{
  This function computes the geodesic distance according to Billera et. al.
  using an algorithm based off of the polynomial time algorithm of Owen
  and Provan. Since it corresponds to a formal definition of tree-space as
  a space of strictly binary trees, no mulifurcations are allowed, including
  on the root node. In addition, negative and 0-lengthed edges are clamped to
  a very small value (DBL_MIN) for technical reasons.

  The Newick parser supports only a subset of the Newick format. In
  particular, it does not at the moment allow for internal node labels, only
  weights. Weights will be automatically set to 1 if not specified. It may be
  necessary to clean data in ape to make the trees conform to this.
}
\value{
  Returns a distance matrix of class 'dist' representing the pairwise
  geodesic distances between all input trees. Keep in mind this distance
  matrix is not Euclidean. N/A values are provided in the case of an error in
  determining the distance.
}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.

  Billera, L. J., Holmes, S. P., and Vogtmann, K. (2001) Geometry of the space
  of phylogenetic trees. \emph{Advances in Applied Mathematics},
  \bold{27}, 733--767.

  Megan Owen and J. Scott Provan (2010) A fast algorithm for computing
  geodesic distances in tree space. \emph{IEEE/ACM Transactions on
  Computational Biology and Bioinformatics}, 14 Jan. 2010.
}
\author{John Chakerian}
\seealso{
  \code{\link[ape]{dist.dna}}, \code{\link[ape]{boot.phylo}},
  \code{\link[stats]{cmdscale}}
}
\examples{

data(woodmouse)
otree <- root(nj(dist.dna(woodmouse)), "No305", resolve.root=TRUE)
breps <- 250

trees <- boot.phylo(otree, woodmouse, B=breps, function(x)
    root(nj(dist.dna(x)), "No305", resolve.root=TRUE), trees = TRUE)

combined.trees <- c(list(otree), trees$trees)
tree.dists <- dist.multiPhylo(combined.trees)

mdres <- cmdscale(tree.dists, k=breps, add=TRUE)
plot(mdres$points[,1], mdres$points[,2], col = c("red", rep("black", breps)))
text(mdres$points[,1], mdres$points[,2], labels = 1:(breps + 1),
     cex = 0.7, adj = c(0, 2))
}
\keyword{manip}

\name{orthant.boundary.tree}
\Rdversion{1.1}
\alias{orthant.boundary.tree}
\title{Orthant Boundary Tree}
\description{
  Produces a degenerate tree on the boundary between trees that differ by one split.
}
\usage{
orthant.boundary.tree(x,y)
}
\arguments{
  \item{x}{The tree in the first orthant.}
  \item{y}{The tree in the second orthant.}
}
\details{
  The tree found is the tree on the boundary between the two orthants such
  that it is on the straight line connecting the two trees when one
  orthant is thought of as being the (-,+) quadrant and the second
  orthant as being the (+,+) quadrant, where the (0,y) line is the particular
  boundary in question.
}
\value{
  Returns an object of class 'phylo' representing the boundary tree.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{\code{\link{mcmc.target.seq}}}

\examples{

data(woodmouse)
otree <- root(fastme.ols(dist.dna(woodmouse)), "No305", resolve.root=TRUE)
breps <- 200

trees <- boot.phylo(otree, woodmouse, B=breps, function(x)
        root(fastme.ols(dist.dna(x)), "No305", resolve.root=TRUE),
        trees = TRUE)

combined.trees <- c(list(otree), trees$trees)

binning <- bin.multiPhylo(combined.trees)

tree.a <- combined.trees[[match(1, binning)]]
i <- 2
max.bin <- max(binning)
tree.b <- combined.trees[[match(2, binning)]]

while(length(distinct.edges(tree.a,tree.b)) > 1 && i < max.bin)
{
    i = i + 1
    tree.b = combined.trees[[match(i, binning)]]
}

plot(orthant.boundary.tree(tree.a, tree.b))
}
\keyword{manip}

\name{distory-package}
\Rdversion{1.1}
\alias{distory-package}
\alias{distory}
\docType{package}
\title{Distance Between Phylogenetic Histories}
\description{
  The \pkg{distory} package provides functions for computing geodesic distances
  between phylogenetic trees, as well as functions which use this distance.
  Methods for computing Gromov delta-hyperbolicity, Markov Chain Monte Carlo
  routines in tree space, and per-position leverage for DNA sequences are
  included.
}
\details{
  A description of the algorithm used for the distance computation can
  be found in \code{\link{dist.multiPhylo}}.
}
\author{
  John Chakerian <chakj@stanford.edu> and Susan Holmes <susan@stat.stanford.edu>
}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.

  Billera, L. J., Holmes, S. P., and Vogtmann, K. (2001) Geometry of the space
  of phylogenetic trees. \emph{Advances in Applied Mathematics},
  \bold{27}, 733_-767.

  Megan Owen and J. Scott Provan (2010) A fast algorithm for computing
  geodesic distances in tree space. \emph{IEEE/ACM Transactions on
  Computational Biology and Bioinformatics}, 14 Jan. 2010.
}
\keyword{package}

\name{bin.multiPhylo}
\Rdversion{1.1}
\alias{bin.multiPhylo}
\title{Bin Trees}
\description{
  Bins trees according to branching topology.
}
\usage{
bin.multiPhylo(treelist)
}
\arguments{
  \item{treelist}{A list of trees that can be passed to dist.phylo (see
    the help for dist.phylo for acceptable formats).}
}
\details{
  Bins trees according to branching topology. Two trees are considered to have
  the same topology if the same set of partitions of tips are produced by
  the edges, which corresponds to the same branching up to rearrangement
  of tips.
}
\value{
  Returns a numeric vector of bin ids. Bin ids are assigned in order of the
  first tree in that bin, that is, the first k unique trees in the list
  passed will be assigned bins 1..k in order of appearance.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{\code{\link{dist.multiPhylo}}}
\examples{
data(woodmouse)
otree <- root(fastme.ols(dist.dna(woodmouse)), "No305", resolve.root=TRUE)
breps <- 500

trees <- boot.phylo(otree, woodmouse, B=breps, function(x)
        root(fastme.ols(dist.dna(x)), "No305", resolve.root=TRUE),trees=TRUE)

combined.trees <- c(list(otree), trees$trees)

bin.multiPhylo(combined.trees)
}
\keyword{manip}

\name{gromov.hyperbolicity}
\Rdversion{1.1}
\alias{gromov.hyperbolicity}
\title{Gromov Hyperbolicity Constant}
\description{
  Computes the Gromov Hyperbolicity Constant of a distance matrix.
}
\usage{
gromov.hyperbolicity(d, deltas = FALSE, scale = NA)
}
\arguments{
  \item{d}{
    A distance matrix of type \code{dist} or \code{matrix}, or anything that
    can be coerced into \code{dist} by \code{as.dist}. Must have at least 4
    points.}
  \item{deltas}{
    A logical value specifying whether to return the vector of delta values.
    Default is \code{FALSE}.}
  \item{scale}{
    Specifies a scaling method for each delta. Default is no scaling (NA or
    "none"). Available methods are "max" which scales deltas by the max of the
    sums computed, and "perimeter" for the largest perimeter of the four
    points.}
}
\details{
  This computes a constant that represents the relaxation of a 4-point
  condition for delta-hyperbolicity. See (Gromov 1987) for details.
}
\value{
  The Gromov hyperbolicity constant of the given distance matrix.
}
\author{John Chakerian}
\seealso{\code{\link{dist.multiPhylo}}}
\references{
  M. Gromov. \emph{Hyperbolic groups}. In Essays in Group Theory, pages
  73--263. Springer, New York, 1987.

  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\examples{
# scale final delta by max distance
points <- cbind(runif(100), runif(100))
d <- dist(points)
gromov.hyperbolicity(d)/max(d)

# scale each delta by max distance for the 4 points
points <- cbind(runif(100), runif(100))
d <- dist(points)
gromov.hyperbolicity(d, scale="max")

# scale each delta by the max perimeter for the 4 points
points <- cbind(runif(100), runif(100))
d <- dist(points)
gromov.hyperbolicity(d, scale="max")
}
\keyword{manip}

\name{phylo.diff}
\Rdversion{1.1}
\alias{phylo.diff}
\alias{distinct.edges}
\alias{edge.from.split}
\alias{get.bipartition}
\alias{partition.leaves}
\title{Differences Between Phylogenetic Trees}
\description{
  A family of functions for determining and plotting the differences between
  two trees.

  \code{phylo.diff} plots two trees side by side, highlighting edges unique
  to each tree in red.

  \code{distinct.edges} finds the edges present in the first argument not in
  the second.

  \code{edge.from.split} locates the edge id from a given split.

  \code{get.bipartition} gets the bipartition of tips formed by a single edge.

  \code{partition.leaves} returns the set of all bipartitions from all edges.
}
\usage{
phylo.diff(x, y, \dots)

distinct.edges(x, y)

edge.from.split(x, split)

get.bipartition(x, e)

partition.leaves(x)
}
\arguments{
  \item{x}{The first (or only) tree.}
  \item{y}{The second tree, for the functions that accept two trees.}
  \item{split}{A list of bipartitions, probably from \code{partition.leaves}.}
  \item{e}{An edge for a particular tree, given as an id.}
  \item{\dots}{Additional arguments to pass to the \code{plot.phylo} function.}
}
\details{
  \code{phylo.diff} uses the ape tree plotting function. The other functions
  are mostly meant as support functions.
}
\value{
  \code{phylo.diff} returns invisible.

  \code{distinct.edges} returns a numeric vector of edge ids for the first
  tree.

  \code{edge.from.split} returns an edge id for a particular tree
  corresponding to a given bipartition and NA if none such edge exists.

  \code{get.bipartition} returns a character vector of the tips below that
  edge in the given tree.

  \code{partition.leaves} returns a list of partitions (themselves character
  vectors) of the given tree.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{\code{\link{dist.multiPhylo}}}
\examples{
data(woodmouse)
otree <- root(fastme.ols(dist.dna(woodmouse)), "No305", resolve.root=TRUE)
breps <- 10

trees <- boot.phylo(otree, woodmouse, B=breps, function(x)
        root(fastme.ols(dist.dna(x)), "No305", resolve.root=TRUE),
        trees = TRUE)

combined.trees <- c(list(otree), trees$trees)

binning <- bin.multiPhylo(combined.trees)

phylo.diff(combined.trees[[match(1, binning)]], combined.trees[[match(2, binning)]])
}
\keyword{manip}

\name{bethe.tree}
\Rdversion{1.1}
\alias{bethe.tree}
\title{Bethe Tree}
\description{
  Generates a Bethe tree with given tips, inner edge lengths, and outgroup.
}

\usage{
bethe.tree(tips, level.lengths = NULL, outgroup="O", outgroup.dist=1)
}

\arguments{
  \item{tips}{
    A list of tip names as a character vector. Should be a power of 2. All
    tip names must be distinct.}
  \item{level.lengths}{
    Edge lengths for each level, counted from the bottom up. NULL means a
    default of 1. If the vector isn't long enough, the last value will be
    repeated as necessary.}
  \item{outgroup}{The tip label for the outgroup.}
  \item{outgroup.dist}{The distance of the outgroup from the root.}
}
\details{
  Generates a Bethe tree with specified internal edge lengths.
}
\value{
  A class of type \code{phylo} representing the tree.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{\code{\link{dist.multiPhylo}}}
\examples{
plot(bethe.tree(as.character(1:16), 1:4, "17", 14))
}
\keyword{datagen}

\name{position.leverage}
\Rdversion{1.1}
\alias{position.leverage}
\title{Position Leverage}
\description{
  Provides a rough heuristic for determining the degree to which each
  position in the DNA matrix affects the resulting tree.
}
\usage{
position.leverage(data, F, to = NULL, rep = 50, by = 1)
}
\arguments{
  \item{data}{A DNA matrix in \code{DNAbin} format.}
  \item{F}{
    A tree estimation function, accepting a DNA matrix of class \code{DNAbin}
    and returning a tree of class \code{phylo}.}
  \item{to}{
    The tree with which distances are measured in respect to, or \code{NULL}
    to indicate the tree estimated by \code{F} for the starting DNA matrix.}
  \item{rep}{
    The number of times to replicate the position in question.}
  \item{by}{
    The function will perform the calculation on every \code{by}-th position
    (that is, on \code{seq(1, N, by)}).}}
\details{
  This function takes a DNA matrix and, for every \code{by}-th position,
  replicates that position \code{rep} times, randomly removing
  \code{rep} other positions to keep all sequences the same length
  other positions to keep all sequences the same length. For each new DNA
  matrix created in this way, \code{F} is used to estimate the corresponding
  tree, and the distance to tree \code{to} is computed and stored. This
  distance can be thought of as somewhat analogous to the leverage of that
  position.
}
\value{
  Returns a numeric vector of distances from tree \code{to} for each position
  sampled.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{\code{\link{dist.multiPhylo}}}
\examples{
data(woodmouse)
f.est <- function(x) root(nj(dist.dna(x)), "No305", resolve.root = TRUE)
position.leverage(woodmouse, f.est, by = 10)
}
\keyword{manip}

\name{mcmc.target.seq}
\Rdversion{1.1}
\alias{mcmc.target.seq}
\alias{boot.samples.idxs}
\alias{lookup.samples}
\alias{convert.table.to.idx}
\title{Find MCMC Target Sequence}

\description{
  \code{mcmc.target.seq} uses MCMC to find a configuration of DNA positions
  to get as close as possible to a given tree.

  \code{boot.samples.idxs} bootstraps over indices into a DNA matrix.

  \code{lookup.samples} goes from an index representation of a configuration
  of DNA to the actual DNAbin format.

  \code{convert.table.to.idx} converts a table of counts for positions 1..n
  into a list of indices corresponding to positions (i.e. goes from the
  tabled form to a vector whose tabling matches the input).
}
\usage{
mcmc.target.seq(data, x, F, n)

boot.samples.idxs(data, B = 100, block = 1)

lookup.samples(data, idxs)

convert.table.to.idx(T)
}
\arguments{
  \item{data}{A DNA matrix in DNAbin format.}
  \item{x}{A tree of class 'phylo' to estimate.}
  \item{F}{A tree estimation function, accepting a DNA matrix in DNAbin
    format and returning a tree of class 'phylo.'}
  \item{n}{The number of MCMC iterations to perform.}
 \item{B}{ The number of bootstrap replicates.}
 \item{block}{The block size to use during bootstrapping.}
 \item{idxs}{A list of numeric vectors of indices to use for lookup.}
 \item{T}{A table or table-like vector to convert.}
}
\details{
  \code{mcmc.target.seq} performs an MCMC with simulated annealing to locate
  a configuration of DNA positions from the original matrix that gets as
  close as possible to a target tree. Propositions for the MCMC replacing
  one character with another uniformly at random.

  The remaining functions are intended to be used as support functions.
}
\value{
  \code{mcmc.target.seq} returns a list of 4 elements: a numeric vector of
  counts of each position in the original matrix, the best estimated
  tree, the best distance from the estimated tree to the target tree, and
  a numeric vector of the distances for every iteration of the
  simulation.

  \code{boot.samples.idxs} returns a numeric vector representing the
  bootstrapped idices.

  \code{lookup.samples} returns a list of objects of class DNAbin
  corresponding to the DNA sequences generated from indices into the original
  DNA matrix.

  \code{convert.table.to.idx} returns a numeric vector of indices based on
  the table counts.
}
\author{John Chakerian}
\references{
  Chakerian, J. and Holmes, S. P. Computational Tools for Evaluating
  Phylogenetic and Heirarchical Clustering Trees. arXiv:1006.1015v1.
}
\seealso{
  \code{\link{dist.multiPhylo}}, \code{\link{orthant.boundary.tree}}
}

\examples{
\dontrun{
## This example has been excluded from checks:
## copy/paste the code to try it

data(woodmouse)
otree <- root(fastme.ols(dist.dna(woodmouse)), "No305", resolve.root=TRUE)
breps <- 200

trees <- boot.phylo(otree, woodmouse, B=breps, function(x)
        root(fastme.ols(dist.dna(x)), "No305", resolve.root=TRUE),
        trees = TRUE)

combined.trees <- c(list(otree), trees$trees)

binning <- bin.multiPhylo(combined.trees)

tree.a <- combined.trees[[match(1, binning)]]
i <- 2
max.bin <- max(binning)
tree.b <- combined.trees[[match(2, binning)]]

while(length(distinct.edges(tree.a,tree.b)) > 1 && i < max.bin)
{
    i = i + 1
    tree.b = combined.trees[[match(i, binning)]]
}

bdy.tree <- orthant.boundary.tree(tree.a, tree.b)

f.est <- function(x) root(nj(dist.dna(x)), "No305", resolve.root=TRUE)

res <- mcmc.target.seq(woodmouse, bdy.tree, f.est, 1000)

par(mfrow=c(2,1))
plot(res$tree)
plot(res$vals)
}}
\keyword{manip}
